<?php

namespace Bixo\Company\Http\Controllers;

use Exception;
use Bixo\Company\Actions\CompanyWorkflow;
use Bixo\Company\Http\Requests\CompanyWorkflowRequest;
use Bixo\Company\Http\Resources\CompanyResource;
use Bixo\Company\Models\Company;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class CompanyWorkflowController extends BaseController
{
    /**
     * Action controller function for company.
     *
     * @param Model $company
     * @param action next action for the company.
     *
     * @return Response
     */
    public function __invoke(CompanyWorkflowRequest $request, Company $company, $trasition)
    {
        try {
            $request = $request->all();
            $company = CompanyWorkflow::run($trasition, $company, $request);
            $data = new CompanyResource($company);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company::company.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('company/company/' . $company->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/company/' . $company->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the company and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $company
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $company, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $company = Company::findBySignedId($company);
        $transitions = $this->transitions($company, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('company::company.name') . ' ' . trans('Approvals'))
            ->view('company::public.company.workflow')
            ->layout('mobile')
            ->data(compact('company', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the company.
     *
     * @param PublicRequest $request
     * @param string $company
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $company, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $company = Company::findBySignedId($company);
        $user = user();
        $company = CompanyWorkflow::run($transition, $company, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($company, $user)
    {
        $transitions = [];
        foreach ($company->workflow()->transitions($company) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.company.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'company' => $company->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $company->workflow()->form($value);
            $array['label'] = trans('company::company.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
